@@ -27,9 +27,9 @@ class ClockSettingDialog : public QDialog, private Ui::ClockSettingDialog {
 
 private slots:
   void scalingSliderHandler(int value);
+  void noAnimationCheckboxHandler(bool checked);
 
  public:
-
   /*!
     \brief コンストラクタ
 
@@ -38,6 +38,7 @@ private slots:
   explicit ClockSettingDialog(QDialog* parent = 0);
   ~ClockSettingDialog(void);
 
+
   /*!
     \brief 拡大率のスライドバーを設定
 
@@ -45,14 +46,18 @@ private slots:
   */
   void setScalingPercent(int value);
 
-signals:
+  void setNoAnimationCheckbox(bool checked);
 
+
+signals:
   /*!
     \brief 拡大率の変更
 
     \param percent [o] 拡大率 [%]
   */
   void scalingValueChanged(int percent);
+
+  void noAnimationStateChanged(bool checked);
 };
 
 #endif /* !SETTING_DIALOG_FORM_H */
