@@ -149,6 +149,7 @@ struct SettingConfig::pImpl {
     // デジタル画像
     tags_.insert("digital/parts", "digital");
     tags_.insert("digital/colon", "on");
+    tags_.insert("digital/blink", "on");
     tags_.insert("digital/sec", "on");
 
     tags_.insert("digital/ap_xpos", "0");
@@ -273,7 +274,7 @@ struct SettingConfig::pImpl {
     // デジタル情報
     const char* digital_tags[] = {
       "parts",
-      "colon", "sec",
+      "colon", "blink", "sec",
       "ap_xpos", "ap_ypos",
       "h1_xpos", "h1_ypos", "h2_xpos", "h2_ypos", "cl_xpos", "cl_ypos",
       "m1_xpos", "m1_ypos", "m2_xpos", "m2_ypos",
@@ -306,6 +307,9 @@ struct SettingConfig::pImpl {
     n = sizeof(additional_tags) / sizeof(additional_tags[0]);
     writeTagValues(fout, "additional", additional_tags, n);
 
+    // アニメーション情報
+    writeAnimationTags(fout);
+
     fout << "<lm_tmclock" << std::endl;
 
     return true;
@@ -329,6 +333,87 @@ struct SettingConfig::pImpl {
 
     fout << "  </" << base << ">" << std::endl;
   }
+
+
+  // アニメーション情報
+  void writeAnimationTags(std::ofstream& fout) {
+
+    fout << "  <animation>" << std::endl;
+
+    // !!! ThemeConfigWindow と共通にすべき
+    enum {
+      AnimationCellMax = 999,   // アニメーション用セルの最大数
+    };
+
+    // セル情報
+    for (size_t i = 0; i < AnimationCellMax; ++i) {
+      char name[] = "animation/cell_000/cell";
+      snprintf(name, strlen(name) + 1, "animation/cell_%03d/cell", i);
+
+      TagMap::iterator p = tags_.find(name);
+      if (p == tags_.end()) {
+        break;
+      }
+
+      // 書き出し処理
+      char tag_name[] = "cell_000";
+      snprintf(tag_name, strlen(tag_name) + 1, "cell_%03d", i);
+      fout << "    <" << tag_name << ">" << std::endl;
+      fout << "      <cell> " << p.value() << " </cell>" << std::endl;
+
+      char x_tag[] = "animation/cell_000/xpos";
+      char y_tag[] = "animation/cell_000/ypos";
+      size_t length = strlen(x_tag) + 1;
+      snprintf(x_tag, length, "animation/cell_%03d/xpos", i);
+      snprintf(y_tag, length, "animation/cell_%03d/ypos", i);
+
+      fout << "      <xpos> " << tags_[x_tag] << " </xpos>" << std::endl;
+      fout << "      <ypos> " << tags_[y_tag] << " </ypos>" << std::endl;
+
+      fout << "    </" << tag_name << ">" << std::endl;
+    }
+    fout << std::endl;
+
+    // パターン情報
+    for (size_t i = 0; i < AnimationCellMax; ++i) {
+
+      // !!! 指定したタグがあるか、を判定する関数を作るべきかも
+      char name[] = "animation/pattern_000/pattern";
+      snprintf(name, strlen(name) + 1, "animation/pattern_%03d/pattern", i);
+
+      TagMap::iterator p = tags_.find(name);
+      if (p == tags_.end()) {
+        break;
+      }
+
+      // 書き出し処理
+      char tag_name[] = "pattern_000";
+      snprintf(tag_name, strlen(tag_name) + 1, "pattern_%03d", i);
+
+      std::string probability_tag =
+        std::string("animation/") + tag_name + "/probability";
+      double probability = atof(tags_[probability_tag].c_str());
+
+      std::string cycle_tag =
+        std::string("animation/") + tag_name + "/cycle";
+      size_t cycle = atoi(tags_[cycle_tag].c_str());
+
+      std::string pattern_tag =
+        std::string("animation/") + tag_name + "/pattern";
+      std::string pattern = tags_[pattern_tag];
+
+      fout << "    <" << tag_name << ">" << std::endl;
+      fout << "      <probability> " << probability << " </probability>"
+           << std::endl;
+      fout << "      <cycle> " << cycle << " </cycle>" << std::endl;
+      fout << "      <pattern> " << pattern << " </pattern>" << std::endl;
+
+      fout << "    </" << tag_name << ">" << std::endl;
+      fout << std::endl;
+    }
+
+    fout << "  </animation>" << std::endl;
+  }
 };
 
 
@@ -360,7 +445,7 @@ void SettingConfig::setValue(const std::string& tag, const std::string& value) {
 
   TagMap::iterator p = pimpl->tags_.find(tag.c_str());
   if (p == pimpl->tags_.end()) {
-    // 新しいタグは登録させない
+    // 存在しないタグは、登録させない
     return;
   }
 
