@@ -8,7 +8,7 @@
 
   \todo エラーメッセージの処理
   \todo 時刻毎のベース画像切替え機能を実装する
-  \todo アニメーション機能を付加する
+  \todo アニメーションの設定変更を反映させる
 */
 
 #include <QLabel>
@@ -17,12 +17,15 @@
 #include <QBitmap>
 #include <QCloseEvent>
 #include <QPainter>
+#include <QTimer>
 #include "ThemeClockDraw.h"
 #include "RrdaEncoder.h"
 #include "RrdaResource.h"
 #include "SettingConfig.h"
 #include "ThemeResource.h"
-#include "ClockPartsDraw.h"
+#include "AnimationPattern.h"
+#include "AnimationManager.h"
+#include "SplitString.h"
 
 #ifdef _MSC_VER
 #define snprintf _snprintf
@@ -31,13 +34,16 @@
 
 struct ThemeClockDraw::pImpl {
 
+  enum {
+    AnimationCycleMsec = 100,
+  };
+
   static const bool CenterPosition = true; // 画像中心を基準にして描画
 
   QLabel* message_label_;
   bool theme_loaded_;
   QString theme_file_;
   ThemeResource theme_resource_;
-  QRegion region_;
   QWidget* parent_widget_;
   QVector<QPixmap> digital_pixmap_;
   QVector<QPixmap> number_pixmap_;
@@ -45,13 +51,21 @@ struct ThemeClockDraw::pImpl {
   QVector<QPixmap> separator_pixmap_;
   QVector<QPoint> digital_positions_;
   QVector<QPoint> calendar_positions_;
+  QVector<QPoint> cell_positions_;
   size_t scaling_percent_;
+  bool no_animation_;
   int sec_offset_;
 
+  AnimationManager* animation_manager_;
+  size_t cell_max_;
+  size_t pattern_max_;
+  std::vector<AnimationPattern> patterns_;
+  QTimer animation_cycle_timer_;
 
   pImpl(ThemeClockDraw* parent)
     : message_label_(NULL), theme_loaded_(false), parent_widget_(NULL),
-      scaling_percent_(100), sec_offset_(0) {
+      scaling_percent_(100), no_animation_(false), sec_offset_(0),
+      animation_manager_(NULL), cell_max_(0), pattern_max_(0) {
 
     // テーマファイルのドラッグを促すフォームを作成
     message_label_ =
@@ -63,6 +77,10 @@ struct ThemeClockDraw::pImpl {
     QVBoxLayout* layout = new QVBoxLayout(parent);
     layout->addWidget(message_label_);
     parent->setLayout(layout);
+
+    // アニメーション用
+    connect(&animation_cycle_timer_, SIGNAL(timeout()),
+            parent, SLOT(evaluateAnimation()));
   }
 
 
@@ -144,6 +162,7 @@ struct ThemeClockDraw::pImpl {
     // 設定の格納
     loadOption(resource, "analog/sec", tags);
     loadOption(resource, "digital/colon", tags);
+    loadOption(resource, "digital/blink", tags);
     loadOption(resource, "digital/sec", tags);
     loadOption(resource, "calendar/month", tags);
 
@@ -164,6 +183,9 @@ struct ThemeClockDraw::pImpl {
       return false;
     }
 
+    // アニメーション情報の読み出し
+    loadAnimations(resource, tags, rrda_encoder);
+
     std::swap(theme_resource_, resource);
 
     return true;
@@ -332,16 +354,89 @@ struct ThemeClockDraw::pImpl {
   }
 
 
+  // アニメーション情報の読み出し
+  void loadAnimations(ThemeResource& resource,
+                      QMap<std::string, std::string>& tags,
+                      RrdaEncoder& rrda_encoder) {
+
+    // !!! 内部実装を、適当に関数化すること
+
+    // !!! どこかで、どうにかしてくくる
+    enum {
+      AnimationCellMax = 1000,   // アニメーション用セルの最大数
+      AnimationPatternMax = 1000,   // アニメーション用パターンの最大数
+    };
+
+    for (size_t i = 0; i < AnimationCellMax; ++i) {
+      // タグが存在するかで枚数をカウント
+      char tag_name[] = "animation/cell_000/cell";
+      snprintf(tag_name, strlen(tag_name) + 1, "animation/cell_%03d/cell", i);
+
+      if (! loadPixmap(resource, tag_name, tags, rrda_encoder)) {
+        break;
+      }
+
+      // セル位置の読み出し
+      char x_tag[] = "animation/cell_000/xpos";
+      char y_tag[] = "animation/cell_000/ypos";
+      size_t length = strlen(x_tag) + 1;
+      snprintf(x_tag, length, "animation/cell_%03d/xpos", i);
+      snprintf(y_tag, length, "animation/cell_%03d/ypos", i);
+
+      int x = atoi(tags[x_tag].c_str());
+      int y = atoi(tags[y_tag].c_str());
+      cell_positions_.push_back(QPoint(x, y));
+
+      // !!! とりあえず
+      // !!! いずれ、整合を取る
+      resource.offset.insert(tag_name, QRect(x, y, 0, 0));
+
+      cell_max_ = i + 1;
+    }
+
+    // アニメーションパターンの読み出し
+    for (size_t i = 0; i < AnimationPatternMax; ++i) {
+      // 順番に評価して、見つからなければ、終了
+      char tag_name[] = "animation/pattern_000/pattern";
+      snprintf(tag_name, strlen(tag_name) + 1,
+               "animation/pattern_%03d/pattern", i);
+
+      QMap<std::string, std::string>::iterator p = tags.find(tag_name);
+      if (p == tags.end()) {
+        break;
+      }
+
+      // パターン情報の登録
+      // !!! qtmclock_config にも同じコードがある。いずれ、くくる
+      AnimationPattern animation;
+
+      char base_name[] = "pattern_000";
+      snprintf(base_name, strlen(base_name) + 1, "pattern_%03d", i);
+
+      std::string probability_name =
+        std::string("animation/") + base_name + "/probability";
+      animation.probability = atof(tags[probability_name].c_str());
+
+      std::string cycle_name = std::string("animation/") + base_name + "/cycle";
+      size_t cycle_msec = atoi(tags[cycle_name].c_str());
+      animation.cycle_msec = cycle_msec;
+
+      std::string pattern = std::string("animation/") + base_name + "/pattern";
+      std::vector<std::string> tokens;
+      size_t n = qrk::split(tokens, tags[pattern], ", \t");
+      for (size_t j = 0; j < n; ++j) {
+        animation.cells.push_back(atoi(tokens[j].c_str()));
+      }
+      patterns_.push_back(animation);
+
+      pattern_max_ = i + 1;
+    }
+  }
+
+
   // フォームの初期化
   void initializeWidget(ThemeClockDraw* parent) {
 
-#if 0
-    // 透明にしてみる
-    //parent->setAutoFillBackground(true);
-    QColor color(0, 0, 0, 0);
-    parent->setPalette(color);
-#endif
-
     resizeWidget(parent);
   }
 
@@ -362,18 +457,11 @@ struct ThemeClockDraw::pImpl {
   // 再描画
   void paintEvent(ThemeClockDraw* parent) {
 
-    // 部品の再描画
     QPainter painter(parent);
-    region_ = QRegion();
-    ClockPartsDraw::redraw(painter, region_, theme_resource_,
-                           digital_positions_, calendar_positions_,
-                           scaling_percent_, sec_offset_);
-    if (parent_widget_) {
-      parent_widget_->setMask(region_);
-    }
-    painter.end();
+    ClockPartsDraw::drawPixmaps(painter, true);
   }
 
+
   // テーマのリソースからの読み出し
   bool loadThemeResouce(const QMap<std::string, QPixmap*>& pixmaps,
                         const QMap<std::string, QRect>& offset,
@@ -420,6 +508,20 @@ bool ThemeClockDraw::loadThemeFile(const QString& theme_file,
   pimpl->initializeWidget(this);
   pimpl->message_label_->hide();
 
+  // アニメーション管理オブジェクトの初期化
+  delete pimpl->animation_manager_;
+  if (pimpl->pattern_max_ == 0) {
+    pimpl->animation_manager_ = NULL;
+    pimpl->animation_cycle_timer_.stop();
+
+  } else {
+    pimpl->animation_manager_ =
+      new AnimationManager(pImpl::AnimationCycleMsec, pimpl->patterns_);
+    pimpl->animation_cycle_timer_.start(pImpl::AnimationCycleMsec);
+    connect(pimpl->animation_manager_, SIGNAL(updateCell(int)),
+            this, SLOT(updateCell(int)));
+  }
+
   return true;
 }
 
@@ -456,12 +558,28 @@ const QString ThemeClockDraw::getThemeFile(void) {
 
 
 // 再描画
-void ThemeClockDraw::redraw(int sec_offset, QWidget* parent) {
+void ThemeClockDraw::redraw(int sec_offset, QWidget* parent, int cell_index) {
 
   pimpl->sec_offset_ = sec_offset;
   pimpl->parent_widget_ = parent;
   if (pimpl->theme_loaded_) {
-    // 描画イベントを発生させる
+
+    // 描画する pixmap を評価し、mask を生成する
+    QRegion region;
+    ClockPartsDraw::updatePixmaps(region,
+                                  pimpl->theme_resource_,
+                                  pimpl->digital_positions_,
+                                  pimpl->calendar_positions_,
+                                  pimpl->scaling_percent_,
+                                  sec_offset, cell_index);
+
+    // mask を適用する。mask によっては、この時点で paintEvent() が呼ばれる
+    if (parent) {
+      parent->setMask(region);
+    }
+
+    // paintEvent() が呼ばれなかったとき用
+    // もし、setMask() 時に paintEvent() が呼ばれていた場合、処理はすぐに戻る
     update();
   }
 }
@@ -491,3 +609,28 @@ void ThemeClockDraw::updateScalingPercent(size_t percent) {
   pimpl->scaling_percent_ = percent;
   pimpl->resizeWidget(this);
 }
+
+
+void ThemeClockDraw::setAnimationOn(bool on) {
+
+  pimpl->no_animation_ = ! on;
+
+  // 現在のアニメーション設定を更新する
+  // !!!
+}
+
+
+// アニメーション開始の評価
+void ThemeClockDraw::evaluateAnimation(void) {
+
+  if (pimpl->animation_manager_) {
+    pimpl->animation_manager_->evaluate();
+  }
+}
+
+
+// 表示セルの更新
+void ThemeClockDraw::updateCell(int index) {
+
+  redraw(pimpl->sec_offset_, pimpl->parent_widget_, index);
+}
