@@ -11,6 +11,7 @@
 */
 
 #include <QWidget>
+#include "ClockPartsDraw.h"
 
 
 /*!
@@ -25,7 +26,12 @@ class ThemeClockDraw : public QWidget {
   struct pImpl;
   std::auto_ptr<pImpl> pimpl;
 
+private slots:
+  void evaluateAnimation(void);
+  void updateCell(int index);
+
 protected:
+
   /*!
     \brief 描画イベント処理
 
@@ -33,6 +39,7 @@ protected:
   */
   void paintEvent(QPaintEvent* event);
 
+
   /*!
     \brief Qt のクローズイベント
 
@@ -47,7 +54,6 @@ public:
     \param parent [i] 親ウィジット
   */
   ThemeClockDraw(QWidget* parent = 0);
-
   ~ThemeClockDraw(void);
 
 
@@ -65,6 +71,8 @@ public:
 
   /*!
     \brief テーマのリソースからの読み出し
+
+    qtmclock_config のプレビュー表示用
   */
   bool loadThemeResource(const QMap<std::string, QPixmap*>& pixmaps,
                          const QMap<std::string, QRect>& offset,
@@ -87,8 +95,11 @@ public:
     \brief テーマ時計の再描画を行う
 
     \param sec_offset [i] 描画時間のオフセット [秒]
+    \param parent [i] 親ウィジット
+    \param cell_index [i] 描画するアニメーションセル
   */
-  void redraw(int sec_offset = 0, QWidget* parent = NULL);
+  void redraw(int sec_offset = 0, QWidget* parent = NULL,
+              int cell_index = ClockPartsDraw::InvalidCellIndex);
 
 
   /*!
@@ -106,6 +117,8 @@ public:
   */
   void updateScalingPercent(size_t percent);
 
+  void setAnimationOn(bool on);
+
 signals:
   /*!
     \brief 終了シグナルを投げる
