@@ -32,14 +32,17 @@ struct ThemeClockWidget::pImpl {
   QString theme_file_;
   QPoint drag_position_;
   size_t scaling_percent_;
+  bool no_animation_;
   size_t pre_scaling_percent_;
+  bool pre_no_animation_;
 
   ClockSettingDialog setting_dialog_;
 
 
   pImpl(ThemeClockWidget* parent)
     : draw_widget_(new ThemeClockDraw(parent)),
-      scaling_percent_(100), pre_scaling_percent_(100) {
+      scaling_percent_(100), pre_scaling_percent_(100),
+      pre_no_animation_(false) {
   }
 
   // フォームの初期化
@@ -55,6 +58,10 @@ struct ThemeClockWidget::pImpl {
     connect(&setting_dialog_, SIGNAL(scalingValueChanged(int)),
             parent, SLOT(changeScaling(int)));
 
+    // アニメーションの On / Off
+    connect(&setting_dialog_, SIGNAL(noAnimationStateChanged(bool)),
+            parent, SLOT(changeNoAnimation(bool)));
+
     // テーマファイルの Drop を許可
     parent->setAcceptDrops(true);
 
@@ -68,8 +75,8 @@ struct ThemeClockWidget::pImpl {
   // 次に秒針が動くまでのミリ秒だけ待機
   void startClock(ThemeClockWidget* parent) {
 
-    QTime current_time = QTime::currentTime();
     clock_timer_.stop();
+    QTime current_time = QTime::currentTime();
 
     QTimer::singleShot(1000 - current_time.msec(), parent, SLOT(redraw()));
   }
@@ -97,9 +104,15 @@ struct ThemeClockWidget::pImpl {
       return true;
 
     } else if (loadThemeFile(pre_theme_file_, parent)) {
+
       // 前回利用したテーマ。拡大率があれば、設定する
       parent->changeScaling(pre_scaling_percent_);
       setting_dialog_.setScalingPercent(scaling_percent_);
+
+      // アニメーションの指定があれば、設定する
+      parent->changeNoAnimation(pre_no_animation_);
+      setting_dialog_.setNoAnimationCheckbox(scaling_percent_);
+
       return true;
 
     } else {
@@ -134,8 +147,8 @@ struct ThemeClockWidget::pImpl {
 
     QSettings settings("Hyakuren Soft LTD.", "qtmclock");
     pre_theme_file_ = settings.value("theme_file", "").toString();
-
     pre_scaling_percent_ = settings.value("scaling_percent", 0).toInt();
+    pre_no_animation_ = settings.value("no_animation", false).toBool();
 
     if (! position_specified) {
       // 位置が指定されてないときのみ、設定を反映させる
@@ -153,6 +166,7 @@ struct ThemeClockWidget::pImpl {
     settings.setValue("theme_file", fi.absoluteFilePath());
 
     settings.setValue("scaling_percent", scaling_percent_);
+    settings.setValue("no_animation", no_animation_);
 
     QRect position = parent->geometry();
     QSize size = parent->size();
@@ -249,7 +263,6 @@ void ThemeClockWidget::mouseMoveEvent(QMouseEvent* event) {
 // テーマのリロード用
 void ThemeClockWidget::dragEnterEvent(QDragEnterEvent* event) {
 
-  setBackgroundRole(QPalette::Highlight);
   event->acceptProposedAction();
 }
 
@@ -278,9 +291,8 @@ void ThemeClockWidget::dropEvent(QDropEvent* event) {
     path.replace(0, 1, "");
 
     // 日本語のパスを含む場合への対処
-    path = toStdStringPath(path);
-
-    if (pimpl->loadThemeFile(path, this)) {
+    std::string std_path = toStdStringPath(path);
+    if (pimpl->loadThemeFile(QString(std_path.c_str()), this)) {
       return;
 
     } else {
@@ -308,3 +320,10 @@ void ThemeClockWidget::changeScaling(int percent) {
   pimpl->draw_widget_->updateScalingPercent(percent);
   resize(pimpl->draw_widget_->size());
 }
+
+
+void ThemeClockWidget::changeNoAnimation(bool checked) {
+
+  pimpl->no_animation_ = checked;
+  pimpl->draw_widget_->setAnimationOn(pimpl->no_animation_);
+}
