@@ -37,12 +37,24 @@ public:
 extern void loadDigitalPosition(QVector<QPoint>& digital_positions,
                                 const QMap<std::string, std::string>& tags);
 
+/*!
+  \overload
+*/
 extern void loadDigitalPosition(QVector<QPoint>& digital_positions,
                                 QMap<std::string, QPoint>& parts_positions);
 
+/*!
+  \brief カレンダー部品位置の設定読み出し
+
+  \param calendar_positions [i]
+  \param tags [i]
+*/
 extern void loadCalendarPosition(QVector<QPoint>& calendar_positions,
                                  const QMap<std::string, std::string>& tags);
 
+/*!
+  \overload
+*/
 extern void loadCalendarPosition(QVector<QPoint>& calendar_positions,
                                  QMap<std::string, QPoint>& parts_positions);
 
