@@ -17,10 +17,12 @@ class ArchiveConfig;
   \brief リリースデータ生成
 
   \param archive_config [i] アーカイブ管理オブジェクト
+  \param cell_max [i] アニメーションのセル枚数
 
   \retval true 正常
   \retval false エラー
 */
-extern bool releaseDataFile(const ArchiveConfig& archive_config);
+extern bool releaseDataFile(const ArchiveConfig& archive_config,
+                            size_t cell_max);
 
 #endif /* !RELEASE_DATA_H */
