@@ -16,6 +16,7 @@
 #include <QShortcut>
 #include <QMessageBox>
 #include <QTimer>
+#include <QSettings>
 #include <cmath>
 #include "ThemeConfigWindow.h"
 #include "ArchiveConfig.h"
@@ -26,6 +27,8 @@
 #include "ThemeClockDraw.h"
 #include "ThemeResource.h"
 #include "PartsPlaceScene.h"
+#include "CellPreviewWidget.h"
+#include "PatternRegisterWidget.h"
 
 using namespace qrk;
 
@@ -117,20 +120,20 @@ struct ThemeConfigWindow::pImpl {
   std::vector<QPixmap> separator_parts_;
 
   PositionMap parts_positions_;
-
   ReleaseDialog release_dialog_;
   std::string work_dir_;
-
   ThemeClockDraw preview_draw_;
-
   QTimer redraw_timer_;
-
   QCursor cursor_;
+  size_t cell_max_;
+  size_t pattern_max_;
+  std::vector<PatternRegisterWidget*> pattern_register_;
 
   pImpl(void)
     : new_act_(NULL), open_act_(NULL), save_act_(NULL),
       archive_config_(NULL), setting_config_(NULL), save_ok_(false),
-      same_hands_center_(false), scene_(NULL), work_dir_(".") {
+      same_hands_center_(false), scene_(NULL), work_dir_("."),
+      cell_max_(0), pattern_max_(0) {
   }
 
 
@@ -151,11 +154,14 @@ struct ThemeConfigWindow::pImpl {
     parent->parts_tree_->header()->setResizeMode(1, QHeaderView::Stretch);
 
     // タブの初期選択
+    parent->mode_tab_->setCurrentIndex(0);
     parent->checkbox_tab_->setCurrentIndex(0);
 
     // チェックボックス
     connect(parent->center_checkbox_, SIGNAL(clicked(bool)),
             parent, SLOT(centerCheckHandler(bool)));
+    connect(parent->show_colon_checkbox_, SIGNAL(clicked(bool)),
+            parent, SLOT(showColonCheckHandler(bool)));
     connect(parent->blink_checkbox_, SIGNAL(clicked(bool)),
             parent, SLOT(blinkCheckHandler(bool)));
     connect(parent->analog_checkbox_, SIGNAL(clicked(bool)),
@@ -190,6 +196,8 @@ struct ThemeConfigWindow::pImpl {
     // 部品表示ツリー
     connect(parent->parts_tree_, SIGNAL(itemSelectionChanged()),
             parent, SLOT(imageUploadHandler()));
+    connect(parent->parts_tree_, SIGNAL(itemClicked(QTreeWidgetItem*, int)),
+            parent, SLOT(updateAnimationItems(QTreeWidgetItem*, int)));
 
     // リリース
     connect(&release_dialog_, SIGNAL(release(QString, QString)),
@@ -325,6 +333,9 @@ struct ThemeConfigWindow::pImpl {
     QTreeWidgetItem* near = additional->child(0);
     registerTreeItem("additional/near", near);
 
+    // アニメーション
+    QTreeWidgetItem* animation = parent->parts_tree_->itemBelow(additional);
+    registerTreeItem("animation", animation);
   }
 
 
@@ -420,6 +431,8 @@ struct ThemeConfigWindow::pImpl {
   // 読み出し
   void load(ThemeConfigWindow* parent) {
 
+    // !!! cell_max_, pattern_max_ をどこかで初期化すべき
+
     // 設定を読み出して反映する
     loadSettings(parent);
 
@@ -445,14 +458,32 @@ struct ThemeConfigWindow::pImpl {
     bool digital_sec = ! setting_config_->getValue("digital/sec").compare("on");
     parent->digital_checkbox_->setChecked(digital_sec);
 
+    // コロンの 表示 / 非表示
+    bool colon = ! setting_config_->getValue("digital/colon").compare("on");
+    parent->show_colon_checkbox_->setChecked(colon);
+
+    // コロンの点滅 する / しない
+    bool blink = ! setting_config_->getValue("digital/blink").compare("on");
+    parent->digital_checkbox_->setChecked(blink);
+
     // カレンダー月の 表示 / 非表示
     bool calendar_month =
       ! setting_config_->getValue("calendar/month").compare("on");
     parent->month_checkbox_->setChecked(calendar_month);
 
+    // アニメーション情報の読み出し
+    loadAnimations();
+
     // 部品画像のツリーへの設定
     loadImages();
 
+    // アニメーションパターンのタブへの登録
+    loadAnimationPatterns(parent);
+    if (cell_max_ == 0) {
+      // アニメーション情報がなければ、配置タブを開く
+      parent->mode_tab_->setCurrentIndex(0);
+    }
+
     return true;
   }
 
@@ -471,19 +502,19 @@ struct ThemeConfigWindow::pImpl {
 
 
   // 画像の読み出し
-  void loadImage(const char* tag) {
+  bool loadImage(const char* tag) {
 
     const std::string archive_tag = setting_config_->getValue(tag);
     const std::string name = archive_config_->getNormalData(archive_tag);
     if (name.empty()) {
-      return;
+      return false;
     }
 
     // 現在の画像を破棄し、新規に画像を読み込む
     delete pixmaps_.value(tag, NULL);
     delete icons_.value(tag, NULL);
 
-    QPixmap* pixmap = new QPixmap(fromStdStringPath(name));
+    QPixmap* pixmap = new QPixmap(toQtStringPath(name));
     pixmaps_[tag] = pixmap;
 
     QIcon* icon = new QIcon(*pixmap);
@@ -511,8 +542,135 @@ struct ThemeConfigWindow::pImpl {
     } else if (! strcmp(tag, "calendar/week")) {
       updateWeekParts(*pixmap);
     }
+
+    return true;
   }
 
+
+  // アニメーション情報の読み出し
+  void loadAnimations(void) {
+
+    enum {
+      AnimationCellMax = 1000,   // アニメーション用セルの最大数
+      AnimationPatternMax = 1000,   // アニメーション用パターンの最大数
+    };
+
+    // アニメーション用の画像を読み出す
+    QTreeWidgetItem* animation_item = name_to_item_["animation"];
+    for (size_t i = 0; i < AnimationCellMax; ++i) {
+      // 順番に評価して、見つからなければ、終了
+      char tag_name[] = "animation/cell_000/cell";
+      snprintf(tag_name, strlen(tag_name) + 1, "animation/cell_%03d/cell", i);
+
+      // アニメーション登録用のツリーを追加
+      char tree_name[] = "cell_000";
+      snprintf(tree_name, strlen(tree_name) + 1, "cell_%03d", i);
+      QTreeWidgetItem* cell_item = new QTreeWidgetItem(animation_item);
+      cell_item->setText(0, tree_name);
+      cell_item->setCheckState(0, Qt::Unchecked);
+      registerTreeItem(tag_name, cell_item);
+
+      if (! loadImage(tag_name)) {
+        break;
+      }
+      cell_max_ = i + 1;
+    }
+
+    // アニメーション用画像の位置を、反映させる
+    for (size_t i = 0; i < cell_max_; ++i) {
+      char tag_name[] = "animation/cell_000/cell";
+      snprintf(tag_name, strlen(tag_name) + 1, "animation/cell_%03d/cell", i);
+
+      char x_tag[] = "animation/cell_000/xpos";
+      char y_tag[] = "animation/cell_000/ypos";
+      size_t length = strlen(x_tag) + 1;
+      snprintf(x_tag, length, "animation/cell_%03d/xpos", i);
+      snprintf(y_tag, length, "animation/cell_%03d/ypos", i);
+
+      int x = atoi(setting_config_->getValue(x_tag).c_str());
+      int y = atoi(setting_config_->getValue(y_tag).c_str());
+      parts_positions_[tag_name] = QPoint(x, y);
+    }
+
+    // パターン情報の読み出し
+    for (size_t i = 0; i < AnimationPatternMax; ++i) {
+      // 順番に評価して、見つからなければ、終了
+      char tag_name[] = "animation/pattern_000/pattern";
+      snprintf(tag_name, strlen(tag_name) + 1,
+               "animation/pattern_%03d/pattern", i);
+      if (setting_config_->getValue(tag_name).empty()) {
+        break;
+      }
+
+      // 情報が抜けていたら、無効
+      // !!! とりあえず、省略
+
+      pattern_max_ = i + 1;
+    }
+  }
+
+
+  // アニメーション情報の保存
+  void saveAnimations(void) {
+
+    for (size_t i = 0; i < cell_max_; ++i) {
+
+      // 位置の記録
+      char name[] = "animation/cell_000/cell";
+      snprintf(name, strlen(name) + 1, "animation/cell_%03d/cell", i);
+
+      char x_tag[] = "animation/cell_000/xpos";
+      char y_tag[] = "animation/cell_000/ypos";
+      size_t length = strlen(x_tag) + 1;
+      snprintf(x_tag, length, "animation/cell_%03d/xpos", i);
+      snprintf(y_tag, length, "animation/cell_%03d/ypos", i);
+
+      // XML ファイルへの記録
+      position_tag_t tag;
+      tag.name = name;
+      tag.x_tag = x_tag;
+      tag.y_tag = y_tag;
+      saveEachPosition(tag);
+    }
+  }
+
+
+  // アニメーションパターン登録用のウィジットを配置
+  void loadAnimationPatterns(ThemeConfigWindow* parent) {
+
+    QWidget* widget_parent = parent->mode_tab_->widget(1);
+
+    pattern_register_.clear();
+    for (size_t i = 0; i < pattern_max_; ++i) {
+
+      PatternRegisterWidget* widget = new PatternRegisterWidget(widget_parent);
+
+      // 設定パターンを反映
+      char base_name[] = "pattern_000";
+      snprintf(base_name, strlen(base_name) + 1, "pattern_%03d", i);
+
+      widget->setName(base_name);
+
+      std::string probability_name =
+        std::string("animation/") + base_name + "/probability";
+      double probability =
+        atof(setting_config_->getValue(probability_name).c_str());
+      widget->setProbability(probability);
+
+      std::string cycle_name = std::string("animation/") + base_name + "/cycle";
+      size_t cycle = atoi(setting_config_->getValue(cycle_name).c_str());
+      widget->setCycle(cycle);
+
+      std::string pattern =
+        std::string("animation/") + base_name + "/pattern";
+      widget->setPattern(setting_config_->getValue(pattern));
+
+      parent->patterns_layout_->addWidget(widget);
+      pattern_register_.push_back(widget);
+    }
+  }
+
+
   // 数値部品の更新
   void updateDigitalParts(const QPixmap& pixmap) {
 
@@ -613,6 +771,17 @@ struct ThemeConfigWindow::pImpl {
     QGraphicsItem* parent = NULL;
     parent = registerItem(parent, "base/base_org");
 
+    // アニメーション部品
+    for (size_t i = 0; i < cell_max_; ++i) {
+      char tag_name[] = "animation/cell_000/cell";
+      snprintf(tag_name, strlen(tag_name) + 1, "animation/cell_%03d/cell", i);
+
+      if ((! current_tag_.compare("animation")) ||
+          (name_to_item_[tag_name]->checkState(0) == Qt::Checked)) {
+        registerItem(parent, tag_name);
+      }
+    }
+
     // カレンダー部品
     registerItem(parent, "calendar/m1", "C0");
     registerItem(parent, "calendar/m2", "C1");
@@ -633,7 +802,9 @@ struct ThemeConfigWindow::pImpl {
       registerItem(parent, "digital/s2", "5");
     }
 
-    registerItem(parent, "digital/cl", ":");
+    if (! setting_config_->getValue("digital/colon").compare("on")) {
+      registerItem(parent, "digital/cl", ":");
+    }
     registerItem(parent, "digital/ap", "A");
 
     // アナログ部品
@@ -819,7 +990,10 @@ struct ThemeConfigWindow::pImpl {
     if (archive_config_->saveAs(name)) {
 
       // 位置の反映
-      savePosition();
+      savePositions();
+
+      // アニメーション情報の保存
+      saveAnimations();
 
       // XML 設定の保存
       setting_config_->save();
@@ -887,24 +1061,30 @@ struct ThemeConfigWindow::pImpl {
   }
 
 
-  void savePosition(void) {
+  // 部品位置の保存 (全体)
+  void savePositions(void) {
 
     // 描画用の位置を setting_config_ に反映させる
     size_t n = sizeof(PositionTags) / sizeof(PositionTags[0]);
     for (size_t i = 0; i < n; ++i) {
-      const position_tag_t& tag = PositionTags[i];
-
-      QPoint& point = parts_positions_[tag.name];
-
-      QString x = QString("%1").arg(point.x());
-      QString y = QString("%1").arg(point.y());
-
-      setting_config_->setValue(tag.x_tag, x.toStdString().c_str());
-      setting_config_->setValue(tag.y_tag, y.toStdString().c_str());
+      saveEachPosition(PositionTags[i]);
     }
   }
 
 
+  // 部品位置の保存
+  void saveEachPosition(const position_tag_t& tag) {
+
+    QPoint& point = parts_positions_[tag.name];
+
+    QString x = QString("%1").arg(point.x());
+    QString y = QString("%1").arg(point.y());
+
+    setting_config_->setValue(tag.x_tag, x.toStdString().c_str());
+    setting_config_->setValue(tag.y_tag, y.toStdString().c_str());
+  }
+
+
   // 位置を更新
   void updatePosition(ThemeConfigWindow* parent,
                       int x, int y, const std::string& tag) {
@@ -997,6 +1177,18 @@ struct ThemeConfigWindow::pImpl {
       updatePosition(parent, x, y, "digital/s2");
     }
 
+    // アニメーション部品をまとめて移動させる
+    if (! tag.compare("animation")) {
+      for (size_t i = 0; i < cell_max_; ++i) {
+
+        // !!! タグを作る処理は、関数化して、くくるべきか？
+        char tag_name[] = "animation/cell_000/cell";
+        snprintf(tag_name, strlen(tag_name) + 1, "animation/cell_%03d/cell", i);
+        updatePosition(parent, x, y, tag_name);
+      }
+      return;
+    }
+
     PositionMap::iterator p = parts_positions_.find(tag);
     if (p == parts_positions_.end()) {
       // 見つからなければ、戻る
@@ -1022,7 +1214,7 @@ struct ThemeConfigWindow::pImpl {
   // リリースファイルの作成
   void release(void) {
 
-    releaseDataFile(*archive_config_);
+    releaseDataFile(*archive_config_, cell_max_);
   }
 
 
@@ -1084,6 +1276,8 @@ struct ThemeConfigWindow::pImpl {
     option.insert("digital/sec",
                   parent->digital_checkbox_->isChecked() ? "on" : "off");
     option.insert("digital/colon",
+                  parent->show_colon_checkbox_->isChecked() ? "on" : "off");
+    option.insert("digital/blink",
                   parent->blink_checkbox_->isChecked() ? "on" : "off");
     option.insert("calendar/month",
                   parent->month_checkbox_->isChecked() ? "on" : "off");
@@ -1303,10 +1497,18 @@ ThemeConfigWindow::ThemeConfigWindow(const QString& config_file)
   // ツリー構成の取得
   pimpl->treeTagInitialize(this);
 
+  // 最後に、どちらのタブが開かれていたかを読み出す
+  QSettings settings("Hyakuren Soft LTD.", "qtmclock_config");
+  int mode_tab_index = settings.value("mode_tab", 0).toInt();
+  // 前回の設定を反映させる
+  mode_tab_->setCurrentIndex(mode_tab_index);
+
   // ファイルが指定されていれば、読み出し
   if (! config_file.isEmpty()) {
     pimpl->loadArchiveFile(this, config_file.toStdString());
     pimpl->setSaveEnabled(this, false);
+  } else {
+    mode_tab_->setCurrentIndex(0);
   }
 
   // サブツリーを開く
@@ -1315,6 +1517,10 @@ ThemeConfigWindow::ThemeConfigWindow(const QString& config_file)
 
 
 ThemeConfigWindow::~ThemeConfigWindow(void) {
+
+  // 最後に、どちらのタブが開かれていたかを記録する
+  QSettings settings("Hyakuren Soft LTD.", "qtmclock_config");
+  settings.setValue("mode_tab", mode_tab_->currentIndex());
 }
 
 
@@ -1350,7 +1556,7 @@ void ThemeConfigWindow::openFile(void) {
 
   QString fileName =
     QFileDialog::getOpenFileName(this, tr("Open archive list file."),
-				 fromStdStringPath(pimpl->work_dir_.c_str()),
+				 toQtStringPath(pimpl->work_dir_.c_str()),
                                  tr("archive list (*.txt)"));
 
   if (! fileName.isEmpty()) {
@@ -1359,9 +1565,9 @@ void ThemeConfigWindow::openFile(void) {
     pimpl->preview_draw_.close();
 
     // 日本語のパスを含む場合への対処
-    fileName = toStdStringPath(fileName);
+    std::string std_path = toStdStringPath(fileName);
 
-    pimpl->loadArchiveFile(this, fileName.toStdString());
+    pimpl->loadArchiveFile(this, std_path);
     pimpl->setSaveEnabled(this, false);
   }
 }
@@ -1390,14 +1596,14 @@ void ThemeConfigWindow::saveFileAs(void) {
 
   QString fileName =
     QFileDialog::getSaveFileName(this, tr("Save archive list file."),
-				 fromStdStringPath(pimpl->work_dir_.c_str()),
+				 toQtStringPath(pimpl->work_dir_.c_str()),
                                  tr("archive list (*.txt)"));
 
   if (! fileName.isEmpty()) {
     // 日本語のパスを含む場合への対処
-    fileName = toStdStringPath(fileName);
+    std::string std_path = toStdStringPath(fileName);
 
-    pimpl->save(this, fileName.toStdString().c_str());
+    pimpl->save(this, std_path);
   }
 }
 
@@ -1478,6 +1684,33 @@ void ThemeConfigWindow::aboutApplication(void) {
 }
 
 
+// 描画アイテムの更新
+void ThemeConfigWindow::updateAnimationItems(QTreeWidgetItem* item,
+                                             int column) {
+  static_cast<void>(column);
+
+  std::string name = pimpl->item_to_name_[item];
+  if (! name.compare(0, strlen("animation/cell_"), "animation/cell_")) {
+
+    // 指定セルが、チェックされていなければ、チェックは変更しない
+    if (item->checkState(0) != Qt::Checked) {
+      return;
+    }
+
+    for (size_t i = 0; i < pimpl->cell_max_; ++i) {
+      // 指定セル以外は閉じる
+      char tag_name[] = "animation/cell_000/cell";
+      snprintf(tag_name, strlen(tag_name) + 1, "animation/cell_%03d/cell", i);
+      QTreeWidgetItem* compare_item = pimpl->name_to_item_[tag_name];
+      if (compare_item != item) {
+        compare_item->setCheckState(0, Qt::Unchecked);
+      }
+    }
+    pimpl->updateItems();
+  }
+}
+
+
 // 針の中心を、揃えるか、の設定
 void ThemeConfigWindow::centerCheckHandler(bool checked) {
 
@@ -1487,10 +1720,19 @@ void ThemeConfigWindow::centerCheckHandler(bool checked) {
 }
 
 
+// コロンを表示させるか、の設定
+void ThemeConfigWindow::showColonCheckHandler(bool checked) {
+
+  pimpl->setting_config_->setValue("digital/colon", checked ? "on" : "off");
+  pimpl->updateItems();
+  pimpl->applyCheckChanged(this);
+}
+
+
 // コロンを点滅させるか、の設定
 void ThemeConfigWindow::blinkCheckHandler(bool checked) {
 
-  pimpl->setting_config_->setValue("digital/colon", checked ? "on" : "off");
+  pimpl->setting_config_->setValue("digital/blink", checked ? "on" : "off");
   pimpl->applyCheckChanged(this);
 }
 
@@ -1597,14 +1839,14 @@ void ThemeConfigWindow::updateHandlelr(void) {
 
   QString fileName =
     QFileDialog::getOpenFileName(this, tr("Open parts file."),
-				 fromStdStringPath(pimpl->work_dir_.c_str()),
+				 toQtStringPath(pimpl->work_dir_.c_str()),
                                  tr("parts file (*.png)"));
 
   if (! fileName.isEmpty()) {
     // 日本語のパスを含む場合への対処
-    fileName = toStdStringPath(fileName);
+    std::string std_path = toStdStringPath(fileName);
 
-    pimpl->updateItemName(this, fileName);
+    pimpl->updateItemName(this, std_path.c_str());
   }
 }
 
