@@ -40,6 +40,7 @@ private slots:
   void previewRedraw(void);
 
   void centerCheckHandler(bool checked);
+  void showColonCheckHandler(bool checked);
   void blinkCheckHandler(bool checked);
   void analogCheckHandler(bool checked);
   void digitalCheckHandler(bool checked);
@@ -59,6 +60,7 @@ private slots:
   void offsetUpdated(int value);
 
   void partsClickedHandler(int x, int y);
+  void updateAnimationItems(QTreeWidgetItem* item, int column);
 
 public:
 
